import sys

from status import UNKNOWN, STATUS, OK
from exceptions import StatusNotExist, ParameterInvalid
from perf import PerformanceData


class NagiosPlugin(object):
    """ Nagios plugin

    """

    def __init__(self, name, print_header=True, print_perf=True):
        object.__init__(self)
        self._perf_data = []
        self._status = UNKNOWN
        self._name = name
        self._print_header = print_header
        self._print_perf = print_perf

    @property
    def status(self):
        return self._status

    # @status.setter
    # def status(self, value):
    #     if isinstance(value, int):
    #         if value not in range(0, 3+1):
    #             raise Exception("status accept only values 0, 1, 2, 3")
    #         else:
    #             self._status = value
    #     else:
    #         raise Exception("status muss be a int value")

    def start(self):
        try:
            self.main()
        except Exception as e:
            self.output(UNKNOWN, str(e))

    def main(self):
        self._status, msg_header, msg_perf = self.check()
        msg = "{} {}".format(self._name, STATUS[self.status].lower())
        if self._print_header and msg_header:
            msg += " " + msg_header
        if self._print_perf and msg_perf:
            msg += " | " + msg_perf
        self.output(self.status, msg)

    def add(self, perf_data):
        """ add a performance data

        :param perf_data: PerformanceData object
        :return: nothing
        """
        if not isinstance(perf_data, PerformanceData):
            raise ParameterInvalid("Input data isn't a PerformanceData object")
        self._perf_data.append(perf_data)

    def check(self):
        """ check status of all performance data

        :return: status
        :rtype: int
        """
        if len(self._perf_data) < 1:
            raise Exception("There aren't any data to check")
        status = OK
        msg = ""
        msg_perf = ""
        for perf_data in self._perf_data:
            if msg == "":
                msg = perf_data.str_result()
                msg_perf = perf_data.str_perf()
            else:
                msg += ", " + perf_data.str_result()
                msg_perf += " " + perf_data.str_perf()
            if perf_data.no_check:
                continue
            checked_status = perf_data.check()
            if checked_status > status:
                status = checked_status
            if status == UNKNOWN:
                break
        return status, msg, msg_perf

    @staticmethod
    def output(stt, msg):
        if isinstance(stt, int):
            status = stt
        elif isinstance(stt, str):
            if stt not in STATUS:
                raise StatusNotExist("Status: OK, WARNING, CRITICAL")
            status = STATUS[stt]
        else:
            raise StatusNotExist("Status: OK, WARNING, CRITICAL")
        if msg != '':
            sys.stdout.write("{}\n".format(stt, msg))
        else:
            sys.stdout.write("{}\n".format(stt))
        sys.exit(status)



from exceptions import ParameterInvalid, ValueInvalid
from status import OK, WARNING, CRITICAL


class PerformanceData(object):
    """ Nagios performance data
    """

    def __init__(self, name, value=None, warning=None, critical=None,
                 max_value='', min_value='', unit='', no_check=False,
                 type_check=None):
        object.__init__(self)
        if value is None:
            raise ParameterInvalid(
                "{} need a value".format(self.__class__.__name__))
        try:
            self.__value = int(value)
        except:
            raise ParameterInvalid("value invalid (only accept int or float)")
        self.type_check = type_check

        self.__name = name
        self.__warning = warning
        self.__critical = critical
        self.__max = max_value
        self.__min = min_value
        if unit != '' and unit not in ['s', 'us', 'ms', '%', 'B', 'KB', 'TB',
                                       'GB', 'c']:
            raise Exception("Unit invalid")
        self.__unit = unit
        self.e = None
        self.no_check = no_check

    @property
    def value(self):
        return self.__value

    @value.setter
    def value(self, value):
        try:
            self.__value = float(value)
        except:
            raise ValueInvalid("value must be either int or float")

    def __repr__(self):
        """
        :return: performance data in string
        :rtype: str
        """
        return "{}={}{};{};{};{};{}".format(
            self.__name.strip().replace(' ', '_'), self.value,
            self.__unit, self.__warning,
            self.__critical, self.__min,
            self.__max)

    def str_result(self):
        return "{} = {}{}".format(self.__name, self.value, self.__unit)

    def str_perf(self):
        return self.__repr__()

    def check(self):
        """
        :return: status of this data/counter/gauge after checking
        :rtype: int
        """
        try:
            if self.type_check == "asc":
                if self.__critical is not None and self.__critical <= self.value:
                    return CRITICAL
                if self.__warning is not None and self.__warning <= self.value:
                    return WARNING
                return OK
            elif self.type_check == "desc":
                if self.__critical is not None and self.__critical >= self.value:
                    return CRITICAL
                if self.__warning is not None and self.__warning >= self.value:
                    return WARNING
                return OK
            else:
                return CRITICAL
        except:
            raise



__author__ = 'viet.pham'
__copyright__ = 'GiQ GmbH'
__date__ = '30.07.2015'
__version__ = '0.0.1'
__status__ = 'beta'

from distutils.core import setup

setup(
    name='nagios',
    version='0.0.2',
    packages=['nagios'],
    url='',
    #license='APACHE LICENSE 2.0',
    author='Viet Pham',
    author_email='viet.pham@getgoods.de',
    description='module to develop plugin for nagios, icinga',
    keywords=['nagios', 'icinga', 'plugin'],
)
